package com.nantian.mfp.framework.config;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.access.channel.ChannelProcessingFilter;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.authentication.logout.LogoutFilter;
import org.springframework.security.web.authentication.logout.LogoutHandler;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;
import org.springframework.security.web.authentication.rememberme.PersistentTokenRepository;
import org.springframework.security.web.authentication.session.CompositeSessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.RegisterSessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.SessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.SessionFixationProtectionStrategy;
import org.springframework.security.web.header.HeaderWriterFilter;
import org.springframework.security.web.session.SessionManagementFilter;
import org.springframework.session.FindByIndexNameSessionRepository;
import org.springframework.session.security.SpringSessionBackedSessionRegistry;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nantian.mfp.framework.security.encoder.Sm3PasswordEncoder;
import com.nantian.mfp.framework.security.filter.MfpEwaySafeFilter;
import com.nantian.mfp.framework.security.filter.MfpFlowctrlFilter;
import com.nantian.mfp.framework.security.filter.MfpSessionFilter;
import com.nantian.mfp.framework.security.filter.MfpTxCheckFilter;
import com.nantian.mfp.framework.security.filter.MfpTxXssFilter;
import com.nantian.mfp.framework.security.filter.MfpTxlogFilter;
import com.nantian.mfp.framework.security.filter.MfpUserLoginFilter;
import com.nantian.mfp.framework.security.provider.MfpAuthenticationProvider;
import com.nantian.mfp.framework.security.repository.MfpJdbcTokenRepositoryImpl;
import com.nantian.mfp.framework.security.service.MfpRememberMeServices;
import com.nantian.mfp.framework.security.strategy.MfpSessionControlStrategy;

/**
* 
* @desc  旧版本spring security配置
* @author ganlaifei
* @date   2023/2/21 19:32
*/
@EnableWebSecurity
@Configuration
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Value("${security.login.url:/reCtrl/login}")
    private String LOGIN_URL;

    @Value("${security.login.page:/index.jsp}")
    private String LOGIN_PAGE;

    @Value("${ecurity.logout.url:/reCtrl/logout}")
    private String LOGOUT_URL;

    @Value("${security.login.username:userid}")
    private String LOGIN_USERNAME_PARAMTER;

    @Value("${security.login.password:entry_pwd}")
    private String LOGIN_PASSWORD_PARAMETER;

    @Value("${security.rememberme.token:MFPToken}")
    private String securityRememberMeToken;

    @Value("${security.token.validTime:1209600}")
    private int securityTokenValidTime;

    @Value("${security.remeberme.parameter:autoLogin}")
    private String securityRememberMeParameter;

    @Autowired
    private MfpAuthenticationProvider mfpAuthenticationProvider;

    @Autowired
    private UserDetailsService mfpUserDetailsService;

    @Autowired
    private SavedRequestAwareAuthenticationSuccessHandler mfpAuthSuccessHandler;

    @Autowired
    private SimpleUrlAuthenticationFailureHandler mfpAuthFailureHandler;

    @Autowired
    private LogoutHandler mfpLogoutHandler;

    @Autowired
    private LogoutSuccessHandler mfpLogoutSuccHandler;

    @Autowired
    private FindByIndexNameSessionRepository sessionRepository;

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.authorizeRequests().anyRequest().permitAll()
                .and()
                .formLogin()
                .and()
                .logout()
                .logoutUrl(LOGOUT_URL)
                .addLogoutHandler(mfpLogoutHandler)
                .logoutSuccessHandler(mfpLogoutSuccHandler)
                .permitAll()
                .and()
                .authenticationProvider(mfpAuthenticationProvider)
                .rememberMe()
                .rememberMeServices(mfpRememberMeServices())
                .and()
                .csrf().disable()
                .headers().frameOptions().disable();

        // 日志过滤器,配置为spring security第一个过滤器
        http.addFilterBefore(mfpTxlogFilter(), ChannelProcessingFilter.class);
        // 流控过滤器
        http.addFilterAfter(mfpFlowctrlFilter(), ChannelProcessingFilter.class);
        // 加解密过滤器
        http.addFilterAfter(mfpEwaySafeFilter(), HeaderWriterFilter.class);
        // 交易检查过滤器
        http.addFilterBefore(mfpTxCheckFilter(), LogoutFilter.class);
        // xss过滤器
        http.addFilterAfter(mfpTxXssFilter(), LogoutFilter.class);
        // 登录过滤器
        http.addFilterAt(mfpUserLoginFilter(http.getSharedObject(AuthenticationManager.class)),
                UsernamePasswordAuthenticationFilter.class);
        // session过滤器
        http.addFilterAt(mfpSessionFilter(), SessionManagementFilter.class);
    }

    @Bean
    MfpTxCheckFilter mfpTxCheckFilter(){
        return new MfpTxCheckFilter();
    }

    @Bean
    MfpTxlogFilter mfpTxlogFilter(){
        return new MfpTxlogFilter();
    }

    @Bean
    MfpFlowctrlFilter mfpFlowctrlFilter(){
        return new MfpFlowctrlFilter();
    }

    @Bean
    MfpEwaySafeFilter mfpEwaySafeFilter(){
        return new MfpEwaySafeFilter();
    }

    @Bean
    MfpTxXssFilter mfpTxXssFilter(){
        return new MfpTxXssFilter();
    }

    @Bean
    MfpSessionFilter mfpSessionFilter(){
        return new MfpSessionFilter(sessionRegistry(),LOGIN_PAGE);
    }
    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpFlowctrlFilter> cancelMfpFlowctrlFilter() {
        FilterRegistrationBean<MfpFlowctrlFilter> registration = new FilterRegistrationBean<>(mfpFlowctrlFilter());
        registration.setEnabled(false);
        return registration;
    }

    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpTxlogFilter> cancelMfpTxlogFilter() {
        FilterRegistrationBean<MfpTxlogFilter> registration = new FilterRegistrationBean<>(mfpTxlogFilter());
        registration.setEnabled(false);
        return registration;
    }

    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpTxXssFilter> cancelMfpTxXssFilter() {
        FilterRegistrationBean<MfpTxXssFilter> registration = new FilterRegistrationBean<>(mfpTxXssFilter());
        registration.setEnabled(false);
        return registration;
    }

    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpEwaySafeFilter> cancelMfpEwaySafeFilter() {
        FilterRegistrationBean<MfpEwaySafeFilter> registration = new FilterRegistrationBean<>(mfpEwaySafeFilter());
        registration.setEnabled(false);
        return registration;
    }


    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpTxCheckFilter> cancelMfpTxCheckFilter() {
        FilterRegistrationBean<MfpTxCheckFilter> registration = new FilterRegistrationBean<>(mfpTxCheckFilter());
        registration.setEnabled(false);
        return registration;
    }

    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpSessionFilter> cancelMfpSessionFilter(){
        FilterRegistrationBean<MfpSessionFilter> registration = new FilterRegistrationBean<>(mfpSessionFilter());
        registration.setEnabled(false);
        return registration;
    }

    /**
     * 当filter被spring管理后会自动注册到springmvc过滤器链中,如果再将filter添加到security过滤器链中
     * 会导致filter执行2次,此配置旨在让springboot不自动注册扫面到的filter
     * @return
     */
    @Bean
    FilterRegistrationBean<MfpUserLoginFilter> cancelMfpUserLoginFilter(@Autowired AuthenticationManager authenticationManager){
        FilterRegistrationBean<MfpUserLoginFilter> registration = new FilterRegistrationBean<>(mfpUserLoginFilter(authenticationManager));
        registration.setEnabled(false);
        return registration;
    }



    @Bean
    SessionRegistry sessionRegistry(){
        return new SpringSessionBackedSessionRegistry(sessionRepository);
    }

    /**
     * security session认证策略
     * @returns
     */
    @Bean
    CompositeSessionAuthenticationStrategy compositeSessionAuthenticationStrategy(){
        List<SessionAuthenticationStrategy> list = new ArrayList<SessionAuthenticationStrategy>();
        MfpSessionControlStrategy mfpSessionControlStrategy = new MfpSessionControlStrategy(sessionRegistry());
        mfpSessionControlStrategy.setMaximumSessions(1);
        mfpSessionControlStrategy.setExceptionIfMaximumExceeded(false);
        SessionFixationProtectionStrategy sessionFixationProtectionStrategy = new SessionFixationProtectionStrategy();
        RegisterSessionAuthenticationStrategy registerSessionAuthenticationStrategy = new RegisterSessionAuthenticationStrategy(sessionRegistry());
        list.add(mfpSessionControlStrategy);
        list.add(sessionFixationProtectionStrategy);
        list.add(registerSessionAuthenticationStrategy);
        return new CompositeSessionAuthenticationStrategy(list);
    }



    @Bean
    MfpUserLoginFilter mfpUserLoginFilter(AuthenticationManager authenticationManager){
        MfpUserLoginFilter mfpLoginFilter = new MfpUserLoginFilter();
        mfpLoginFilter.setPasswordParameter(LOGIN_PASSWORD_PARAMETER);
        mfpLoginFilter.setUsernameParameter(LOGIN_USERNAME_PARAMTER);
        mfpLoginFilter.setFilterProcessesUrl(LOGIN_URL);
        mfpLoginFilter.setAuthenticationManager(authenticationManager);
        mfpLoginFilter.setAuthenticationSuccessHandler(mfpAuthSuccessHandler);
        mfpLoginFilter.setAuthenticationFailureHandler(mfpAuthFailureHandler);
        mfpLoginFilter.setRememberMeServices(mfpRememberMeServices());
        mfpLoginFilter.setSessionAuthenticationStrategy(compositeSessionAuthenticationStrategy());
        return mfpLoginFilter;
    }


    @Bean
    MfpRememberMeServices mfpRememberMeServices(){
        MfpRememberMeServices mfpRememberMeServices = new MfpRememberMeServices("mfp",securityTokenValidTime,mfpUserDetailsService,persistentTokenRepository());
        mfpRememberMeServices.setParameter(securityRememberMeParameter);
        mfpRememberMeServices.setCookieName(securityRememberMeToken);
        return mfpRememberMeServices;
    }

    @Bean
    PersistentTokenRepository persistentTokenRepository() {
        MfpJdbcTokenRepositoryImpl tokenRepository = new MfpJdbcTokenRepositoryImpl();
        return tokenRepository;
    }


    @Bean
    PasswordEncoder passwordEncoder() {
        return new Sm3PasswordEncoder();
    }

    /**
     * 获取AuthenticationManager（认证管理器），登录时认证使用
     * @param authenticationConfiguration
     * @return
     * @throws Exception
     */
    @Bean
    AuthenticationManager authenticationManager(AuthenticationConfiguration authenticationConfiguration) throws Exception {
        return authenticationConfiguration.getAuthenticationManager();
    }

    @Bean
    ObjectMapper objectMapper(){
        return new ObjectMapper();
    }
}
