package com.nantian.mfp.framework.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nantian.mfp.framework.context.MfpContextHolder;
import com.nantian.mfp.framework.converter.MfpFormHttpMessageConverter;
import com.nantian.mfp.framework.utils.RequestUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.mvc.condition.PatternsRequestCondition;
import org.springframework.web.servlet.mvc.method.RequestMappingInfo;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;
import org.springframework.web.util.pattern.PathPattern;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ganlaifei
 * @desc springmvc配置
 * @date 2023/3/15 21:45
 */
@Configuration
public class WebMvcConfig implements WebMvcConfigurer, CommandLineRunner {

    public Logger logger = LoggerFactory.getLogger(WebMvcConfig.class);

    public static final List<PathPattern> patternList = new ArrayList<>();

    /**
     * 将自定义表单消息转换器添加到springmvc的消息转换器列表
     *
     * @param converters
     */
    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
        MfpFormHttpMessageConverter converter = new MfpFormHttpMessageConverter();
        converter.setObjectMapper(new ObjectMapper());
        converters.add(converter);
    }

    /**
     * 启动时加载扫描到的 PathVariable路径
     */
    @Override
    public void run(String... args) throws Exception {
        try {
            RequestMappingHandlerMapping mapping = MfpContextHolder.getBean(RequestMappingHandlerMapping.class);
            // 获取url与类和方法的对应信息
            Map<RequestMappingInfo, HandlerMethod> map = mapping.getHandlerMethods();
            for (RequestMappingInfo info : map.keySet()) {
                // 获取url
                PatternsRequestCondition pathPatternsCondition = info.getPatternsCondition();
                Set<PathPattern> patterns = pathPatternsCondition.getPatterns()
                        .stream()
                        .map(RequestUtils.defaultPathPatternParserInstance::parse)
                        .collect(Collectors.toSet());
                for (PathPattern pattern : patterns) {
                    Class<? extends PathPattern> aClass = pattern.getClass();
                    Field field = aClass.getDeclaredField("capturedVariableCount");
                    field.setAccessible(true);
                    int num = (int) field.get(pattern);
                    // 不为0说明路径包含@PathVariable变量
                    if (num != 0) {
                        patternList.add(pattern);
                    }
                }
            }
            logger.info("加载到{}个PathVariable路径:{}", Integer.valueOf(patternList.size()), patternList);
        } catch (Exception e) {
            logger.error("加载PathVariable路径失败", e);
        }
    }
}
