package com.nantian.mfp.framework.utils;

import com.alibaba.csp.sentinel.slots.block.degrade.DegradeException;
import com.alibaba.csp.sentinel.slots.block.flow.FlowException;
import com.alibaba.csp.sentinel.slots.block.flow.param.ParamFlowException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nantian.mfp.framework.config.WebMvcConfig;
import com.nantian.mfp.framework.constant.MfpConstants;
import com.nantian.mfp.framework.context.MfpContextHolder;
import com.nantian.mfp.framework.err.BusinessException;
import com.nantian.mfp.framework.err.HttpException;
import com.nantian.mfp.framework.wrappers.NTNetResetRequestWrapper;
import com.nantian.mfp.pub.model.ResultData;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.MessageSource;
import org.springframework.http.MediaType;
import org.springframework.http.server.PathContainer;
import org.springframework.util.StringUtils;
import org.springframework.web.HttpMediaTypeNotAcceptableException;
import org.springframework.web.accept.HeaderContentNegotiationStrategy;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.util.pattern.PathPattern;
import org.springframework.web.util.pattern.PathPatternParser;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class RequestUtils {
	
	public static Log logger = LogFactory.getLog(RequestUtils.class);

	public static boolean isAjaxRequest(HttpServletRequest request){
		HeaderContentNegotiationStrategy header=new HeaderContentNegotiationStrategy();
		try {
			List<MediaType> requestMediaTypes=header.resolveMediaTypes(new ServletWebRequest(request));
			for (MediaType requestMediaType : requestMediaTypes) {
				if(MediaType.APPLICATION_JSON.includes(requestMediaType)){
					return true;
				};
			}
			
		} catch (HttpMediaTypeNotAcceptableException e) {
			e.printStackTrace();
		}
		
		return false;
		
	}
	

	public static void writerOutErr(HttpServletRequest req, HttpServletResponse res, ObjectMapper objectMapper, Exception ex) throws IOException, ServletException {
		
		ResultData resultData=exceptionToResult(ex ,res);
		res.setContentType("application/json;charset=UTF-8");
		objectMapper.writeValue(res.getOutputStream(), resultData);
	}
	

	
	public static ResultData exceptionToResult(Exception ex,HttpServletResponse response){
		logger.error("====>>服务异常:",ex);
		ResultData resultData = new ResultData();
		resultData.setStatus("-1");
		Object[] args=null;
		if(ex instanceof BusinessException){
			BusinessException be=(BusinessException)ex;
			args=be.getArgs();
			resultData.setErrCode(be.getTerrcode());
			response.setStatus(487);
		}else if(ex instanceof HttpException){
			HttpException he=(HttpException)ex;
			args=he.getArgs();
			resultData.setErrCode(he.getTerrcode());
			response.setStatus(he.getStatusCode());
		}else if(ex instanceof ParamFlowException||ex instanceof FlowException){
			ex = new Exception("服务器繁忙,请稍后再试!");
			resultData.setErrCode("FlowException");
			response.setStatus(470);
		}else if(ex instanceof DegradeException){
			ex = new Exception("触发熔断规则,服务降级!");
			resultData.setErrCode("DegradeException");
			response.setStatus(470);
		}else{
			resultData.setErrCode("SystemError");
			response.setStatus(500);
		}
		resultData.setTxcode(MfpContextHolder.getTxcode());
		resultData.setFlowNo(MfpContextHolder.getReqFlowNo());

		MessageSource messageSource = MfpContextHolder.getBean(MessageSource.class);
		//消息转义
		String msg = messageSource.getMessage(resultData.getErrCode(), args, ex.getMessage(), Locale.getDefault());
		logger.error("请求流水号["+resultData.getFlowNo()+"],http协议状态码["+response.getStatus()+"],交易码["+resultData.getTxcode()+"],技术错误码["+resultData.getErrCode()+"],转义后的消息["+msg+"]");
		resultData.setErrMsg(msg);
		//设置错误码和错误信息到请求线程上下文
		MfpContextHolder.setTerrcode(resultData.getErrCode());
		MfpContextHolder.setErrMessage(msg);

		return resultData;
	}

	public static NTNetResetRequestWrapper getRequestWrapper(HttpServletRequest request) {
		if(request instanceof NTNetResetRequestWrapper){
			return (NTNetResetRequestWrapper)request;
		}
		NTNetResetRequestWrapper requestWrapper = null;
		try {
			requestWrapper = new NTNetResetRequestWrapper(request);
		} catch (IOException e) {
			logger.error("包装HttpServletRequest对象异常");
		}
		return requestWrapper;
	}

	/**
	 * 获取交易码,兼容restful风格
	 * @param request
	 * @return
	 */
	public static String getTxcdoe(HttpServletRequest request){
		String txcode = request.getParameter(MfpConstants.TX_CODE);
		//按照restful风格获取交易码
		if(!StringUtils.hasText(txcode)) {
			for (PathPattern pattern : WebMvcConfig.patternList) {
				//使用spring工具类判断当前请求路径是否符合缓存的pathPathPattern
				String requestURI = request.getRequestURI();
				String contextPath = MfpContextHolder.getProps("server.servlet.context-path");
				//根据url截取项目名称
				if(StringUtils.hasText(contextPath)&&!"/".equals(contextPath)){
					requestURI = requestURI.substring(contextPath.length());
				}
				boolean match = pattern.matches(PathContainer.parsePath(requestURI));
				if (match) {
					//解析pathPathPattern uri获取txcode
					PathPattern.PathMatchInfo pathMatchInfo = pattern.matchAndExtract(PathContainer.parsePath(requestURI));
					Map<String, String> uriVariablesMap = pathMatchInfo.getUriVariables();
					return uriVariablesMap.get(MfpConstants.TX_CODE);
				}
			}
		}
		return txcode;
	}

	public static final PathPatternParser defaultPathPatternParserInstance = new PathPatternParser() {

		@Override
		public void setMatchOptionalTrailingSeparator(boolean matchOptionalTrailingSeparator) {
			raiseError();
		}

		@Override
		public void setCaseSensitive(boolean caseSensitive) {
			raiseError();
		}

		private void raiseError() {
			throw new UnsupportedOperationException(
					"This is a read-only, shared instance that cannot be modified");
		}
	};

}
