package com.nantian.mfp.pub.service;

import com.nantian.mfp.framework.config.DbSequenceConfig;
import com.nantian.mfp.framework.context.MfpContextHolder;
import com.nantian.mfp.pub.dao.SequenceDao;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.UUID;

/**
* 
* @desc  数据库sequence生成工具
* @author ganlaifei
* @date   2023/4/7 21:33
*/
@Service
@ConditionalOnBean(DbSequenceConfig.class)
public class DbSequenceService {

    @Autowired
    SequenceDao sequenceDao;

    static Log log= LogFactory.getLog(SequenceService.class);

    /**
     * 交易流水号
     * @return
     */
    public static String genTxFlowNo() {
        String txFlowNo= UUID.randomUUID().toString();
        MfpContextHolder.setTxFlowNo(txFlowNo);
        return txFlowNo;
    }
    /**
     * 交易流水号
     * @return
     */
    public String getTxFlowNo() {
        return (String) MfpContextHolder.getTxFlowNo();
    }
    public String getGlobalFlowNo() {
        return sequenceDao.generate("全局事件跟踪号", "_globalFlowNo", 10, 1, "GL{yyyy}{mm}{dd}", "");
    }

    public String getTablePK(String tableName, String colunm) {
        return sequenceDao.generate("数据库表格的某个字段的序号","_"+tableName+"_"+colunm , 10, 1, "TB", "");
    }

    /**
     * 生成16位流水号,其中8位时间戳{yyyy}{mm}{dd},后八位为按日复位的递增序号如00000001,00000002,返回的序号为 2015032000000001等
     * @param tableName
     * @param colunm
     * @return 返回的流水号 其中8位时间戳{yyyy}{mm}{dd},后八位为按日复位的递增序号如00000001,00000002,返回的序号为 2015032000000001等
     */
    public String getTableFlowNo(String tableName, String colunm) {
        return sequenceDao.generate("数据库表格的某个字段的序号","_"+tableName+"_"+colunm , 8, 4, "{yyyy}{mm}{dd}", "");
    }

    public String getTableSquence(String tableName, String colunm) {
        return sequenceDao.generate("数据库表格的某个字段的序号","_"+tableName+"_"+colunm , 10, 1, "0", "");
    }

    public List<String> getTableBatchSquence(String tableName, String colunm, int count) {
        return sequenceDao.batchGenerate("数据库表格的某个字段的序号","_"+tableName+"_"+colunm , 10, 1, "0", "",count);
    }


    public  String getReqFlowNo() {
        return (String) MfpContextHolder.getReqFlowNo();
    }

    public  String genReqFlowNo(){
        String reqFlowNo=getGlobalFlowNo();
        MfpContextHolder.setReqFlowNo(reqFlowNo);
        return reqFlowNo;

    }
    public String generate(String seqDefid) {
        return sequenceDao.generate(seqDefid);
    }

    public List<String> batchGenerate(String seqDefid, int count) {
        return sequenceDao.batchGenerate(seqDefid, count);
    }
}
